@@ -1,4 +1,4 @@
-// Copyright (C) 2020-2021 Joel Rosdahl and other contributors
+// Copyright (C) 2020-2023 Joel Rosdahl and other contributors
 //
 // See doc/AUTHORS.adoc for a complete list of contributors.
 //
@@ -21,20 +21,22 @@
 class Context;
 class Hash;
 
-#include "Digest.hpp"
-
-#include "third_party/nonstd/optional.hpp"
-#include "third_party/nonstd/string_view.hpp"
-
+#include <optional>
 #include <string>
+#include <string_view>
 #include <vector>
 
 namespace Depfile {
 
-std::string escape_filename(nonstd::string_view filename);
-nonstd::optional<std::string> rewrite_paths(const Context& ctx,
-                                            const std::string& file_content);
+std::string escape_filename(std::string_view filename);
+
+std::optional<std::string> rewrite_source_paths(const Context& ctx,
+                                                std::string_view file_content);
+
 void make_paths_relative_in_output_dep(const Context& ctx);
-std::vector<std::string> tokenize(nonstd::string_view file_content);
+
+// Tokenize `file_content` into a list of files, where the first token is the
+// target and ends with a colon.
+std::vector<std::string> tokenize(std::string_view file_content);
 
 } // namespace Depfile
