@@ -18,9 +18,7 @@
 
 #include "compopt.hpp"
 
-#include "fmtmacros.hpp"
-
-#include "third_party/fmt/core.h"
+#include <util/fmtmacros.hpp>
 
 // The option it too hard to handle at all.
 #define TOO_HARD (1 << 0)
@@ -53,13 +51,14 @@ struct CompOpt
 
 const CompOpt compopts[] = {
   {"--Werror", TAKES_ARG | AFFECTS_COMP},              // nvcc
-  {"--analyze", TOO_HARD},                             // Clang
+  {"--analyzer-output", TOO_HARD},                     // Clang
   {"--compiler-bindir", AFFECTS_CPP | TAKES_ARG},      // nvcc
   {"--compiler-options", AFFECTS_CPP | TAKES_ARG},     // nvcc
   {"--config", TAKES_ARG},                             // Clang
   {"--gcc-toolchain=", TAKES_CONCAT_ARG | TAKES_PATH}, // Clang
-  {"--libdevice-directory", AFFECTS_CPP | TAKES_ARG},  // nvcc
-  {"--output-directory", AFFECTS_CPP | TAKES_ARG},     // nvcc
+  {"--include", AFFECTS_CPP | TAKES_ARG | TAKES_CONCAT_ARG | TAKES_PATH},
+  {"--libdevice-directory", AFFECTS_CPP | TAKES_ARG}, // nvcc
+  {"--output-directory", AFFECTS_CPP | TAKES_ARG},    // nvcc
   {"--param", TAKES_ARG},
   {"--save-temps", TOO_HARD},
   {"--save-temps=cwd", TOO_HARD},
@@ -101,6 +100,17 @@ const CompOpt compopts[] = {
   {"-all_load", AFFECTS_COMP},
   {"-analyze", TOO_HARD}, // Clang
   {"-arch", TAKES_ARG},
+  {"-ast-dump", AFFECTS_COMP},                         // Clang
+  {"-ast-dump-all", AFFECTS_COMP},                     // Clang
+  {"-ast-dump-all=", AFFECTS_COMP | TAKES_CONCAT_ARG}, // Clang
+  {"-ast-dump-decl-types", AFFECTS_COMP},              // Clang
+  {"-ast-dump-filter", AFFECTS_COMP | TAKES_ARG},      // Clang
+  {"-ast-dump-lookups", AFFECTS_COMP},                 // Clang
+  {"-ast-dump=", AFFECTS_COMP | TAKES_CONCAT_ARG},     // Clang
+  {"-ast-list", AFFECTS_COMP},                         // Clang
+  {"-ast-merge", TOO_HARD | TAKES_ARG},                // Clang
+  {"-ast-print", AFFECTS_COMP},                        // Clang
+  {"-ast-view", TOO_HARD},                             // Clang
   {"-aux-info", TAKES_ARG},
   {"-b", TAKES_ARG},
   {"-bind_at_load", AFFECTS_COMP},
@@ -110,6 +120,8 @@ const CompOpt compopts[] = {
   {"-emit-pth", AFFECTS_COMP},         // Clang
   {"-external:I",
    AFFECTS_CPP | TAKES_ARG | TAKES_CONCAT_ARG | TAKES_PATH}, // msvc
+  {"-fmodule-header", TOO_HARD},
+  {"-fmodules-ts", TOO_HARD},
   {"-fno-working-directory", AFFECTS_CPP},
   {"-fplugin=libcc1plugin", TOO_HARD}, // interaction with GDB
   {"-frepo", TOO_HARD},
@@ -154,6 +166,7 @@ const CompOpt compopts[] = {
   {"-trigraphs", AFFECTS_CPP},
   {"-u", TAKES_ARG | TAKES_CONCAT_ARG},
   {"-v", AFFECTS_COMP},
+  {"-wrapper", TAKES_ARG | TOO_HARD},
   {"-z", TAKES_ARG | TAKES_CONCAT_ARG | AFFECTS_COMP},
 };
 
