@@ -1,4 +1,4 @@
-// Copyright (C) 2010-2021 Joel Rosdahl and other contributors
+// Copyright (C) 2010-2023 Joel Rosdahl and other contributors
 //
 // See doc/AUTHORS.adoc for a complete list of contributors.
 //
@@ -18,15 +18,10 @@
 
 #pragma once
 
-#include "system.hpp"
-
 #include "Config.hpp"
 
 #include <string>
 
-// Return whether a filename has a supported source code extension.
-bool supported_source_extension(const std::string& fname);
-
 // Guess the language of `fname` based on its extension and a compiler type.
 // Returns the empty string if the extension is unknown.
 std::string language_for_file(const std::string& fname,
