@@ -18,7 +18,7 @@
 
 #pragma once
 
-#include <Digest.hpp>
+#include <Hash.hpp>
 #include <core/Result.hpp>
 #include <core/StatisticsCounters.hpp>
 #include <core/types.hpp>
@@ -32,6 +32,7 @@
 #include <third_party/nonstd/span.hpp>
 
 #include <cstdint>
+#include <filesystem>
 #include <optional>
 #include <string_view>
 #include <vector>
@@ -54,7 +55,7 @@ struct CompressionStatistics
 
 enum class FileType { result, manifest, raw, unknown };
 
-FileType file_type_from_path(std::string_view path);
+FileType file_type_from_path(const std::filesystem::path& path);
 
 class LocalStorage
 {
@@ -65,24 +66,32 @@ public:
 
   // --- Cache entry handling ---
 
-  std::optional<util::Bytes> get(const Digest& key, core::CacheEntryType type);
+  std::optional<util::Bytes> get(const Hash::Digest& key,
+                                 core::CacheEntryType type);
 
-  void put(const Digest& key,
+  void put(const Hash::Digest& key,
            core::CacheEntryType type,
            nonstd::span<const uint8_t> value,
            bool only_if_missing = false);
 
-  void remove(const Digest& key, core::CacheEntryType type);
+  void remove(const Hash::Digest& key, core::CacheEntryType type);
 
   static std::string get_raw_file_path(std::string_view result_path,
                                        uint8_t file_number);
-  std::string get_raw_file_path(const Digest& result_key,
+  std::string get_raw_file_path(const Hash::Digest& result_key,
                                 uint8_t file_number) const;
 
   void
-  put_raw_files(const Digest& key,
+  put_raw_files(const Hash::Digest& key,
                 const std::vector<core::Result::Serializer::RawFile> raw_files);
 
+  // Clone, hard link or copy a file from `source` to `dest` depending on
+  // settings in `ctx`. If cloning or hard linking cannot and should not be done
+  // the file will be copied instead. Throws `core::Error` on error.
+  void clone_hard_link_or_copy_file(const std::string& source,
+                                    const std::string& dest,
+                                    bool via_tmp_file = false) const;
+
   // --- Statistics ---
 
   void increment_statistic(core::Statistic statistic, int64_t value = 1);
@@ -130,11 +139,11 @@ private:
   struct LookUpCacheFileResult
   {
     std::string path;
-    Stat stat;
+    util::DirEntry dir_entry;
     uint8_t level;
   };
 
-  LookUpCacheFileResult look_up_cache_file(const Digest& key,
+  LookUpCacheFileResult look_up_cache_file(const Hash::Digest& key,
                                            core::CacheEntryType type) const;
 
   std::string get_subdir(uint8_t l1_index) const;
@@ -144,7 +153,7 @@ private:
   StatsFile get_stats_file(uint8_t l1_index, uint8_t l2_index) const;
 
   void move_to_wanted_cache_level(const core::StatisticsCounters& counters,
-                                  const Digest& key,
+                                  const Hash::Digest& key,
                                   core::CacheEntryType type,
                                   const std::string& cache_file_path);
 
@@ -154,7 +163,7 @@ private:
   std::optional<core::StatisticsCounters> increment_files_and_size_counters(
     uint8_t l1_index, uint8_t l2_index, int64_t files, int64_t size_kibibyte);
   std::optional<core::StatisticsCounters> increment_files_and_size_counters(
-    const Digest& key, int64_t files, int64_t size_kibibyte);
+    const Hash::Digest& key, int64_t files, int64_t size_kibibyte);
 
   void perform_automatic_cleanup();
 
@@ -193,7 +202,7 @@ private:
   // in the directory (including any subdirectories). However, the lock does not
   // have to be acquired to update a level 2 stats file since level 2 content
   // size and file count are stored in the parent (level 1) stats file.
-  util::LockFile get_level_2_content_lock(const Digest& key) const;
+  util::LockFile get_level_2_content_lock(const Hash::Digest& key) const;
   util::LockFile get_level_2_content_lock(uint8_t l1_index,
                                           uint8_t l2_index) const;
 };
