@@ -1,4 +1,4 @@
-// Copyright (C) 2020-2021 Joel Rosdahl and other contributors
+// Copyright (C) 2020-2023 Joel Rosdahl and other contributors
 //
 // See doc/AUTHORS.adoc for a complete list of contributors.
 //
@@ -40,7 +40,4 @@ private:
   static size_t m_subdir_counter;
 };
 
-// Change directory to `dir`, throwing Error on failure.
-void check_chdir(const std::string& dir);
-
 } // namespace TestUtil
