@@ -1,4 +1,4 @@
-// Copyright (C) 2010-2020 Joel Rosdahl and other contributors
+// Copyright (C) 2010-2023 Joel Rosdahl and other contributors
 //
 // See doc/AUTHORS.adoc for a complete list of contributors.
 //
@@ -16,37 +16,38 @@
 // this program; if not, write to the Free Software Foundation, Inc., 51
 // Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-#include "../src/Util.hpp"
-#include "../src/fmtmacros.hpp"
-#include "TestUtil.hpp"
-
-#include "third_party/fmt/core.h"
+#include <util/environment.hpp>
+#include <util/filesystem.hpp>
+#include <util/fmtmacros.hpp>
+#include <util/path.hpp>
 
 #define DOCTEST_THREAD_LOCAL // Avoid MinGW thread_local bug
 #define DOCTEST_CONFIG_IMPLEMENT
 #include "third_party/doctest.h"
 
+namespace fs = util::filesystem;
+
 int
 main(int argc, char** argv)
 {
 #ifdef _WIN32
-  Util::setenv("CCACHE_DETECT_SHEBANG", "1");
+  util::setenv("CCACHE_DETECT_SHEBANG", "1");
 #endif
-  Util::unsetenv("GCC_COLORS"); // Don't confuse argument processing tests.
+  util::unsetenv("GCC_COLORS"); // Don't confuse argument processing tests.
 
-  std::string dir_before = Util::get_actual_cwd();
+  std::string dir_before = util::actual_cwd();
   std::string testdir = FMT("testdir/{}", getpid());
-  Util::wipe_path(testdir);
-  Util::create_dir(testdir);
-  TestUtil::check_chdir(testdir);
+  fs::remove_all(testdir);
+  fs::create_directories(testdir);
+  fs::current_path(testdir);
 
   doctest::Context context;
   context.applyCommandLine(argc, argv);
   int result = context.run();
 
   if (result == 0) {
-    TestUtil::check_chdir(dir_before);
-    Util::wipe_path(testdir);
+    fs::current_path(dir_before);
+    fs::remove_all(testdir);
   } else {
     PRINT(stderr, "Note: Test data has been left in {}\n", testdir);
   }
